#include<stdio.h>
#include<malloc.h>
#include<memory.h>
#include<stdlib.h>
#include<string.h>

#ifndef _WIN32
#  define itoa(n,s,r) sprintf(s,"%d",n)
#endif

// 
typedef unsigned char byte;
typedef unsigned int  uint;
typedef long          BOOL;
#define TRUE		1
#define FALSE		0
//    
#define STUB_SIZE	sizeof(char*)

//!   
void Alarm()
{
	fprintf(stderr,"Internal error\n\a");
	exit(1);
}

//!     
template<class T>class Allocator{
public:
	T* New();			//!<   
	void Free(T*);		//!<  
};

//!    
class StrAllocator{
public:
	char *New(char*,int size);	//!<   
	void Free(char*,int size);	//!<  
};


//! 
class Tree{
public:
	struct stat_st{
		int nodes;			//!<   
		int lines;			//!<   
		int slowness;		//!<   
		int words;			//!<    
	};	//!<    
protected:
	struct node_st{
		union {
			char	*txt;				//!<  
			char	stub[STUB_SIZE];	//!<   
		};
		uint       len;					//!<   
		uint     count;					//!<     
		node_st  *next;					//!<     
		node_st *child;					//!<    
	};	//!<  
	const BOOL duplicates;				//!<     
	node_st *tree_root;					//!<  
	char *GetNodeText(node_st*);		//!<   
private:
	StrAllocator     strings;			//!<    
	Allocator<node_st>memory;			//!<     
	void Clear(node_st *node=0);				//!<   
	void FreeNode(node_st*);					//!<  
	node_st *NewNode(char *text,uint);			//!<  
	void NewNodeText(node_st*,char *text,uint);	//!<    
	void FreeNodeText(node_st*);				//!<   
	BOOL isEnd(node_st*);						//!<        
	void LayNode(node_st *,char *,uint count);	//!<  
	void SpliceNode(node_st*,node_st*);			//!<  
	uint GetStat(node_st *node,stat_st*);		//!<    ( )
public:
	Tree();
	~Tree();
	BOOL Insert(char*);			//!<    
	BOOL Remove(char*);			//!<    
	BOOL Find  (char*);			//!<     
	void GetStat(stat_st*);		//!<  
};

//!     
class Test:public Tree{
	char **words;		//!<  
	uint words_count;	//!<    
	void Dump(FILE *file,node_st *node,uint depth=0);
	void Test1();		
	void Test2();		
	void LoadDic(FILE*);			//!<    
public:
	Test();
	~Test();
	void Go(char *filename);		//!<  
	void Dump(FILE *file=stdout);	//!<     
};


template<class T>T* Allocator<T>::New()
{
	return new T;
}

template<class T>void Allocator<T>::Free(T *obj)
{
	delete obj;
}

//! \param text  
//! \param size  
//! \result     (  )
char *StrAllocator::New(char *text,int size)
{
	char *ref=(char*)malloc(size);
	memcpy(ref,text,size);
	return ref;
}

//! \param text  
//! \param size 
void StrAllocator::Free(char *text,int size)
{
	free(text);
}



Tree::Tree():duplicates(FALSE)
{
	tree_root=0;
}


Tree::~Tree()
{
	Clear();
}

//   
void Tree::Clear(node_st *node)
{
	node_st *next;
	while(node) {
		next=node->next;
		Clear(node->child);
		FreeNode(node);
		node=next;
	}
}

//! \param node  
//! \result TRUE       
BOOL Tree::isEnd(node_st *node)
{
	uint count=0;
	if(!node) Alarm();
	node_st *it=node->child;
	while(it) {
		count+=it->count;
		it=it->next;
	}
	if(node->count==count) return FALSE;
	if(duplicates && node->count<count) Alarm();
	if(!duplicates && node->count!=count+1) Alarm();
	return TRUE;
}

//! \param node  
//! \result  
char *Tree::GetNodeText(node_st *node)
{
	if(node->len<=STUB_SIZE) return node->stub;
	return node->txt;
}

//!      (   )
//! \param node  
//! \param text   
//! \param len   
void Tree::NewNodeText(node_st *node,char *text,uint len)
{
	if(len>STUB_SIZE) node->txt=strings.New(text,len);
	else memcpy(node->stub,text,len);
	node->len=len;
}

//! \param node  
void Tree::FreeNodeText(node_st *node)
{
	if(node->len>STUB_SIZE) strings.Free(node->txt,node->len);
}

//!        
//! \param text  
//! \param len   
//! \result  
Tree::node_st *Tree::NewNode(char *text,uint len)
{
	node_st *node=memory.New();
	node->count=1;
	node->child=node->next=0;
	NewNodeText(node,text,len);
	return node;
}

//! \param node  
void Tree::FreeNode(node_st *node)
{
	FreeNodeText(node);
	memory.Free(node);
}

//!      .     ,     .
//! \param node       
//! \param text     
//! \param count       (   )
void Tree::LayNode(node_st *node,char *text,uint diff)
{
	node_st *branch1,*branch2=0;
	if(!diff || diff>=node->len) Alarm();
	uint len=strlen(text);
	char *txt=GetNodeText(node);
	//  
	             branch1=NewNode(txt+diff,node->len-diff);
	if(diff<len) branch2=NewNode(text+diff,len-diff);
	//   
	node_st temp_node=*node;
	NewNodeText(node,txt,diff);
	FreeNodeText(&temp_node);
	//   
	branch1->child=node->child;
	branch1->count=node->count-1;
	branch1->next=branch2;
	node->child=branch1;
}

//!     .
//! \param node1  
//! \param node2   (node2=node1->child)
void Tree::SpliceNode(node_st *node1,node_st *node2)
{
	if(node2->next) Alarm();
	uint len=node1->len+node2->len;
	node_st temp_node=*node1;
//	char *buff=(char*)malloc(len);
	char *buff=(char*)alloca(len);
	memcpy(buff,GetNodeText(node1),node1->len);
	memcpy(buff+node1->len,GetNodeText(node2),node2->len);
	NewNodeText(node1,buff,len);
//	free(buff);
	node1->child=node2->child;
	FreeNodeText(&temp_node);
	FreeNode(node2);
}

//! \param text  
//! \result TRUE    
BOOL Tree::Insert(char *text)
{
	if(!*text) return FALSE;	//      ,  Find  
	if(!duplicates && Find(text)) return FALSE;
	printf("insert: %s\n",text);
	char *line;					//  
	node_st *curr;				//  
	node_st **ref=&tree_root;	//      ( next  child)
	uint len;					//       
	for(;;) {
		curr=*ref;
		if(!curr) break;
		line=GetNodeText(curr);
		if(*line==*text) {	//   
			len=0;
			curr->count++;
			while(text[len] && len<curr->len) {	//   
				if(text[len]!=line[len]) break;
				len++;
			}
			if(len<curr->len) {	//        ,  
				LayNode(curr,text,len);
				return TRUE;
			}
			text+=len;
			if(!*text) return TRUE;	//     , 
			ref=&curr->child;	//     ,  
			continue;
		}
		ref=&curr->next;
	}
	//       ( )
	*ref=NewNode(text,strlen(text));
	return TRUE;
}

//! \param text  
//! \result TRUE    
BOOL Tree::Remove(char *text)
{
	if(!Find(text)) return FALSE;
	printf("remove: %s\n",text);
	char *line;					//  
	node_st *par=0;				//  
	node_st *curr;				//  
	node_st **ref=&tree_root;	//      (next  child)
	for(;;) {
		curr=*ref;
		if(!curr) break;
		line=GetNodeText(curr);
		if(*line==*text) {	//   
			if(strlen(text)<curr->len) Alarm();
			curr->count--;
			text+=curr->len;
			if(!*text) {	//    (    )
				if(!curr->count) { //     ( )
					*ref=curr->next;
					FreeNode(curr);
					curr=par;
				}
				if(curr && curr->child) { // ,        
					if(curr->count==curr->child->count) SpliceNode(curr,curr->child);
				}
				return TRUE;
			}
			par=curr; 
			ref=&curr->child;
			continue;
		}
		ref=&curr->next;
	}
	return TRUE;
}

//! \param text  
//! \result TRUE     
BOOL Tree::Find(char *text)
{
	char *line;	//  
	int len;	//     
	node_st *node=tree_root;
	while(node) {
		line=GetNodeText(node);
		if(*line==*text) {	//   
			len=node->len;
			while(*text && len) {	//   
				if(*line!=*text) break;
				line++;
				text++;
				len--;
			}
			if(len) return FALSE;	//     ,  
			if(*text) {
				node=node->child;	//       ,     
				continue;
			}
			return isEnd(node);	//    ,       
		}
		node=node->next;
	}
	return FALSE;
}


void Tree::GetStat(stat_st *stat)
{
	memset(stat,0,sizeof(stat_st));
	stat->words=GetStat(tree_root,stat);
}


uint Tree::GetStat(node_st *node,stat_st *stat)
{
	uint count=0;
	if(node) stat->nodes++;
	while(node) {
		stat->lines++;
		stat->slowness+=node->len;
		GetStat(node->child,stat);
		node=node->next;
	}
	return count;
}

// <cur here>
//---------------------------------------------------------
//  

uint rand(uint size)
{
	return ((uint)rand()*65535+(uint)rand()*3+(uint)rand())%size;
}



Test::Test()
{
	words=0;
	words_count=0;
}


Test::~Test()
{
	uint n;
	for(n=0; n<words_count; n++) free(words[n]);
	delete[]words;
}

void Test::Dump(FILE *file)
{
	fprintf(file,"[dump]\n");
	if(tree_root) Dump(file,tree_root,0);
	else fprintf(file,"<empty tree>\n");
}

void Test::Dump(FILE *file,node_st *node,uint depth)
{
	uint n;
	char *text;
	while(node) {
		text=GetNodeText(node);
		for(n=0; n<depth; n++) fputc(' ',file);
		for(n=0; n<node->len; n++) fputc(text[n],file);
		fprintf(file,"   :{%d}\n",node->count);
		Dump(file,node->child,depth+node->len);
		node=node->next;
	}
}


void Test::LoadDic(FILE *file)
{
	uint max_size=0;
	uint size=0;
	uint count=0;
	char sym,old_sym;
	char *buff;
	//       
	old_sym=' ';
	while(!feof(file)) {
		sym=fgetc(file);
		if((byte)sym>' ' && (byte)old_sym<=' ') size=0;
		if((byte)sym<' ' && (byte)old_sym>' ') {
			count++;
			if(size>max_size) max_size=size;
		}
		if((byte)sym>' ') size++;
		old_sym=sym;
	}
	rewind(file);
	//  
	words_count=0;
	buff=(char*)alloca(max_size+1);
	words=new char*[count];
	//  
	old_sym=' ';
	while(!feof(file)) {
		if(words_count>=count) break;
		sym=fgetc(file);
		if((byte)sym>' ' && (byte)old_sym<=' ') size=0;
		if((byte)sym<' ' && (byte)old_sym>' ') {
			buff[size]=0;
			words[words_count++]=strdup(buff);
		}
		if((byte)sym>' ') buff[size++]=sym;
		old_sym=sym;
	}
}


void Test::Go(char *filename)
{
	FILE *file=fopen(filename,"rt");
	if(file) {
		LoadDic(file);
		fclose(file);
	}else fprintf(stderr,"File not found\n\a");
	if(!duplicates) Test1();
	if(file)        Test2();
}

void Test::Test1()
{
	uint n,m;
	bool *map;
	char buff[30];
	const int size=100000;
	printf("test1\n");
	map=(bool*)malloc(size); 
	memset(map,false,size);
	for(n=0; n<100000; n++) {
		m=rand(size); itoa(m,buff,10); map[m]=true;
		Insert(buff);
		m=rand(size); itoa(m,buff,10); map[m]=false;
		Remove(buff);
		m=rand(size); itoa(m,buff,10);
		if(Find(buff)) {
			if(!map[m]) Alarm();
		}else{
			if( map[m]) Alarm();
		}
	}
	free(map);
	for(m=0; m<size; m++) {
		itoa(m,buff,10);
		Remove(buff);
	}
	if(tree_root) Alarm();
}

void Test::Test2()
{
	uint n,m;
	enum{none,use,block};
	printf("test2\n");
	byte *map=(byte*)malloc(words_count); memset(map,none,words_count);
	//   
	for(m=0; m<words_count; m++) {
		if(Insert(words[m])) map[m]=use; else 
			map[m]=block;
	}
	//   
	for(m=0; m<words_count; m++) {
		if(map[m]!=use) continue;
		map[m]=none;
		if(!Remove(words[m])) Alarm();
	}
	if(tree_root) Alarm();
	//  
	for(n=0; n<words_count*4; n++) {
		m=rand(words_count);
		if(map[m]==use  && Remove(words[m])) map[m]=none; else
		if(map[m]==none && Insert(words[m])) map[m]=use;
	}
	//    
	for(m=0; m<words_count; m++) {
		if(map[m]==use  && !Find(words[m])) Alarm();
		if(map[m]==none &&  Find(words[m])) Alarm();
	}
	//  
	for(m=0; m<words_count; m++) {
		if(map[m]==use && !Remove(words[m])) Alarm();
	}
	free(map);
	if(tree_root) Alarm();
}



int main()
{
	Test test;
	test.Go("eng.dic");
	return 0;
}





